Imports System.IO
Imports System.Net
Imports System.Text

Public Class frmMain

#Region " Eigen functies "

    ' Converteer de relatieve image verwijzing naar een absolute
    Private Function CorrectImageKoersenPostbank(ByVal strHtml As String)

        Return strHtml.Replace("/ing/", "http://www.postbank.nl/ing/")

    End Function

    ' Centrale functie om de html pagina aan te maken
    Private Function CreateNewsPaper() As String

        Dim strDailyCartoonHtml As String
        Dim strDailyCartoonImage As String

        ' Bepaal Strip van de dag
        strDailyCartoonHtml = GetHtml("http://www.userfriendly.org")
        strDailyCartoonImage = ExtractDailyCartoon(strDailyCartoonHtml)

        Dim strKoersenPostbankHtml As String
        Dim strKoersenPostbankTable As String
        Dim strKoersenPostbankTableCorrect As String

        ' Bepaal Tabel met koersen
        strKoersenPostbankHtml = GetHtml("http://www.postbank.nl/ing/pp/page/stockexchange/quotes/index/pb/0,2830,1859_270652,00.html")
        strKoersenPostbankTable = ExtractKoersenPostbank(strKoersenPostbankHtml)
        ' De HTML bevat relatieve verwijzingen naar afbeeldingen, corrigeer deze
        strKoersenPostbankTableCorrect = CorrectImageKoersenPostbank(strKoersenPostbankTable)

        ' De hele html pagina
        Dim strNewsPaper As String = GetNewsPaperHeader() & _
                                     GetSubjectHeader("Strip van vandaag") & _
                                     strDailyCartoonImage & "<br><br>" & _
                                     GetSubjectHeader("Koersen Postbank") & _
                                     strKoersenPostbankTableCorrect & _
                                     GetNewsPaperFooter()

        Return strNewsPaper

    End Function

    ' Knip alleen de <img ... > tag eruit
    Private Function ExtractDailyCartoon(ByVal strHtml) As String

        ' Bepaal de positie van de tekst: 'Cartoon for '
        Dim intPosCartoonFor As Integer = strHtml.IndexOf("CARTOON FOR ")

        ' Ga op zoek naar de eerste Image tag na 'Cartoon for '
        Dim intPosImageStart As Integer = strHtml.IndexOf("<IMG ", intPosCartoonFor + 1)

        ' Ga op zoek naar de einde markering van deze tag
        Dim intPostImageEnd As Integer = strHtml.IndexOf(">", intPosImageStart + 1)

        ' Knip de Image tag eruit: '<img ................. >
        Return strHtml.Substring(intPosImageStart, intPostImageEnd - intPosImageStart + 1)

    End Function

    ' Wanneer we de source van deze pagina nader bestuderen komen we erachter
    ' dat de Postbank een class 'ContentIntro' gebruikt voor de weegave van
    ' de koersinformatie. De koerinsformatie is opgeslagen in een drietal, 
    ' geneste tabellen, waarbij wij alleen geinteresseerd zijn in 'diepste'
    ' van de drie.
    Private Function ExtractKoersenPostbank(ByVal strHtml As String) As String

        ' Bepaal de positie van de class 'ContentIntro"
        Dim intPosContentIntro As Integer = strHtml.IndexOf("contentintro")

        ' De tabel die ik wil hebben is de derde (geneste) tabel
        Dim intPosTable1Start As Integer = 0
        Dim intPosTable2Start As Integer = 0
        Dim intPosTable3Start As Integer = 0
        Dim intPosTable3End As Integer = 0

        ' Bepaal positie van de derde geneste tabel op basis van de vorigen
        intPosTable1Start = strHtml.IndexOf("<table", intPosContentIntro + 1)
        intPosTable2Start = strHtml.IndexOf("<table", intPosTable1Start + 1)
        intPosTable3Start = strHtml.IndexOf("<table", intPosTable2Start + 1)
        ' Bepaal de eerste eindemarkering
        intPosTable3End = strHtml.IndexOf("</table>", intPosTable3Start + 1)

        ' Knip de tabel uit de totale pagina. We moeten 7 tekens 'extra' knippen
        ' omdat we de beginpositie van '</table>' bepalen en we deze
        ' eindmarkering wel volledig nodig hebben..
        Return strHtml.Substring(intPosTable3Start, intPosTable3End - intPosTable3Start + 1 + 7)

    End Function

    ' Haal een html pagina op vanaf een locatie op internet
    Private Function GetHtml(ByVal strUrl As String) As String

        ' Gebruik een WebClient en StreamReader om pagina te 
        ' openen en in zijn geheel in te lezen
        Dim objWebClient As New WebClient
        Dim objReader As New StreamReader(objWebClient.OpenRead(strUrl), Encoding.UTF8)
        Dim strHtml As String = objReader.ReadToEnd()

        ' Sluit Reader en ruim object op
        objReader.Close()
        objWebClient.Dispose()

        ' Geef de HTML terug
        Return strHtml

    End Function

    ' Retourneer het bovenste deel van de html pagina
    Private Function GetNewsPaperHeader() As String

        Dim strTitle As String = "PC-Active OchtenKrant van " & _
                                  Date.Now.ToShortDateString()

        ' Geef het bovenste gedeelte terug
        Return "<html><head><title>" & _
               strTitle & _
               "</title></head><body>"

    End Function

    ' Retourneer het onderste deel van de html pagina
    Private Function GetNewsPaperFooter() As String

        ' Geef het onderste gedeelte terug
        Return "</body></html>"

    End Function

    ' Retourneer het onderwerp
    Private Function GetSubjectHeader(ByVal strSubject As String) As String

        ' Zet het onderwerp in een tabel. Grote witte letters
        ' tegen een blauwe achtergrond
        Return "<table width=100%><tr><td bgcolor=#0D237A>" & _
                 "<font face='Arial Black' size=5 color=#FFFFFF>" & _
                    strSubject & _
                 "</font></td></tr></table>"

    End Function

    ' Sla de html pagina op
    Private Function SaveNewsPaper(ByVal strFileName As String, _
                                   ByVal strNewsPaper As String) As Boolean

        ' Open het bestand
        Dim objStreamWriter As New StreamWriter(strFileName)

        ' Schrijf de HTML-pagina weg
        objStreamWriter.Write(strNewsPaper)

        ' Sluit het bestand
        objStreamWriter.Close()

        ' We zijn hier, alles okay
        Return True

    End Function

#End Region

#Region " Events - Form, Toolbar en Menubar "

    ' FORM - De applicatie wordt afgesloten
    Private Sub frmMain_FormClosing(ByVal sender As Object, _
                                    ByVal e As FormClosingEventArgs) _
                                    Handles Me.FormClosing

        ' Vraag gebruiker om bevestiging
        If MessageBox.Show("Wilt u PC-Active Ochtendkrant afsluiten?", _
                           "Afsluiten", MessageBoxButtons.YesNo, _
                           MessageBoxIcon.Question, _
                           MessageBoxDefaultButton.Button2) <> _
                                       Windows.Forms.DialogResult.Yes Then

            ' Annuleer 'het closen'
            e.Cancel = True
        End If

    End Sub

    ' FORM - De applicatie wordt gestart
    Private Sub frmMain_Load(ByVal sender As Object, _
                             ByVal e As System.EventArgs) Handles Me.Load

        Dim strFileName As String = Application.StartupPath & _
                                    "Ochtendkrant.html"
        Dim strNewsPaper As String = CreateNewsPaper()

        ' Sla de pagina op... bepaal of het goed gegaan is
        If SaveNewsPaper(strFileName, strNewsPaper) = True Then
            ' Maak URI en ken deze toe aan WebBrowser
            wbrWebBrowser.Url = New Uri(strFileName, UriKind.Absolute)
        End If

    End Sub

    ' WEBBROWSER - De html pagina is volledig geladen
    Private Sub wbrWebBrowser_Navigated(ByVal sender As Object, _
                                        ByVal e As WebBrowserNavigatedEventArgs) _
                                        Handles wbrWebBrowser.Navigated

        ' Kan ik terug?
        If wbrWebBrowser.CanGoBack = True Then
            tsbGoBack.Enabled = True
        Else
            tsbGoBack.Enabled = False
        End If

        ' ... bovenstaande code kan korter!
        tsbGoForward.Enabled = wbrWebBrowser.CanGoForward

        ' Toon <title> van HTML in titelbalk
        Me.Text = wbrWebBrowser.DocumentTitle

        ' Klaar met laden.. pas statusbalk aan
        slbStatus.Text = "..."

    End Sub

    ' WEBBROWSER - De html pagina wordt geladen
    Private Sub wbrWebBrowser_Navigating(ByVal sender As Object, _
                                         ByVal e As WebBrowserNavigatingEventArgs) _
                                         Handles wbrWebBrowser.Navigating

        ' Pas statusbalk aan
        slbStatus.Text = "Bezig met openen " & _
                         e.Url.ToString & "..."

    End Sub

    ' TOOLBAR - Er wordt op 'Vorige' geklikt
    Private Sub tsbGoBack_Click(ByVal sender As System.Object, _
                                ByVal e As System.EventArgs) _
                                Handles tsbGoBack.Click
        ' Blader terug
        wbrWebBrowser.GoBack()

    End Sub

    ' TOOLBAR - Er wordt op 'Volgende' geklikt
    Private Sub tsbGoForward_Click(ByVal sender As System.Object, _
                                   ByVal e As System.EventArgs) _
                                   Handles tsbGoForward.Click
        'Blader vooruit
        wbrWebBrowser.GoForward()

    End Sub

    ' TOOLBAR - Er wordt op 'Vernieuwen' geklikt
    Private Sub tsbRefresh_Click(ByVal sender As System.Object, _
                                 ByVal e As System.EventArgs) _
                                 Handles tsbRefresh.Click
        ' Ververs de pagina
        wbrWebBrowser.Refresh()

    End Sub

    ' MENUBAR - Er wordt gekozen voor Bestand - Afsluiten
    Private Sub mnuFileExit_Click(ByVal sender As System.Object, _
                                  ByVal e As System.EventArgs) _
                                  Handles mnuFileExit.Click

        ' Sluit scherm en ruim resources op
        Me.Close()
        Me.Dispose()

    End Sub

#End Region

End Class
